package com.mcwl.memberCenter.consumer;

import com.mcwl.common.constant.QueueConstants;
import com.mcwl.memberCenter.domain.UserMember;
import com.mcwl.memberCenter.service.UserMemberService;
import com.mcwl.memberCenter.task.UserMemberTask;
import com.rabbitmq.client.Channel;
import org.springframework.amqp.core.Message;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.List;


@Slf4j
@Component
@RequiredArgsConstructor
public class EmptyPointsRemindConsumer {

    private final UserMemberService userMemberService;

    @RabbitListener(queues = QueueConstants.EMPTY_POINTS_REMIND_QUEUE, ackMode = "MANUAL")
    public void emptyPointsRemind(UserMember UserMember, Channel channel, Message message) {
        try {
            // TODO 发送短信提醒用户积分即将清零
            log.info("获取到积分清零提醒的数据:{}", UserMember);
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            log.error("处理积分清零提醒消息时出错: {}", e.getMessage(), e);
            try {
                channel.basicNack(message.getMessageProperties().getDeliveryTag(), false, true);
            } catch (IOException ex) {
                log.error("消息确认失败: {}", ex.getMessage(), ex);
            }
        }
    }


    @RabbitListener(queues = QueueConstants.MEMBER_BILLING_QUEUE, ackMode = "MANUAL")
    public void memberBillingQueue(UserMember userMember, Channel channel, Message message) {
        try {
            // TODO 发送短信提醒用户会员账单，如果支付成功，更新last_payment_date，并重新计算end_date（start_date + 1个月）
            log.info("获取到会员账单的数据:{}", userMember);
        } catch (Exception e) {
            log.error("处理会员账单消息时出错: {}", e.getMessage(), e);
            try {
                channel.basicNack(message.getMessageProperties().getDeliveryTag(), false, true);
            } catch (IOException ex) {
                log.error("消息确认失败: {}", ex.getMessage(), ex);
            }
        }
    }


}
