package com.mcwl.memberCenter.service.impl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.exception.ServiceException;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.memberCenter.domain.Member;
import com.mcwl.memberCenter.domain.UserMember;
import com.mcwl.memberCenter.enums.MemberMenu;
import com.mcwl.memberCenter.mapper.UserMemberMapper;
import com.mcwl.memberCenter.service.MemberService;
import com.mcwl.memberCenter.service.UserMemberService;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
@RequiredArgsConstructor
public class UserMemberServiceImpl extends ServiceImpl<UserMemberMapper, UserMember> implements UserMemberService {

    private final MemberService memberService;

    private final ISysUserService sysUserService;


    public void aaa() {
        LambdaQueryWrapper<Member> lqw = new LambdaQueryWrapper<>();

        Member member = new Member();
        memberService.update(lqw);
    }

    @Override
    public UserMember createUserMember(Long userId, Long memberId, String paymentMethod) {
        if (userId == null) {
            throw new ServiceException("用户不能为空");
        }
        SysUser user = sysUserService.selectUserById(userId);
        if (user == null) {
            throw new ServiceException("用户不存在");
        }
        if (memberId == null) {
            throw new ServiceException("会员不能为空");
        }
        Member member = memberService.getById(memberId);
        if (member == null) {
            throw new ServiceException("会员不存在");
        }
        // 查询用户是否已经存在会员
        LambdaQueryWrapper<UserMember> lqw = new LambdaQueryWrapper<>();
        lqw.eq(UserMember::getUserId, userId)
                // subscriptionStatus不为expired
                .ne(UserMember::getSubscriptionStatus, MemberMenu.MEMBER_CENTER_EXPIRED)
                .orderBy(true, false, UserMember::getEndDate);
        List<UserMember> userMemberList = baseMapper.selectList(lqw);
        UserMember userMember = new UserMember();
        // 设置用户id
        userMember.setUserId(userId);
        // 设置会员id
        userMember.setMemberId(memberId);
        if (userMemberList != null && !userMemberList.isEmpty()) {
            UserMember lastUserMember = userMemberList.get(0);
            Date endDate = lastUserMember.getEndDate();
            // 设置会员开始时间和结束时间
            Date startDate = new Date(endDate.getTime() + 1000L * 60L * 60L * 24L);
            userMember.setStartDate(startDate);
            userMember.setEndDate(new Date(startDate.getTime() + member.getSubscriptionPeriod() * 1000L * 60L * 60L * 24L));
        } else {
            // 用户不存在会员
            // 设置会员开始时间和结束时间
            Date startDate = new Date();
            userMember.setStartDate(startDate);
            userMember.setEndDate(new Date(startDate.getTime() + member.getSubscriptionPeriod() * 1000L * 60L * 60L * 24L));
        }

        // 设置积分
        userMember.setPoints(member.getPoints());
        // 设置订阅状态
        if (member.getSubscriptionPeriod() == null) {
            // 连续包月，会员状态为活跃
            userMember.setSubscriptionStatus(MemberMenu.MEMBER_CENTER_ACTIVE);
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(userMember.getEndDate());
            calendar.add(Calendar.MONTH, 1);
            userMember.setNextBillingDate(calendar.getTime());
        } else {
            userMember.setSubscriptionStatus(MemberMenu.MEMBER_CENTER_INACTIVE);
        }
        // 设置支付时间
        userMember.setLastPaymentDate(new Date());
        // 设置支付方式
        userMember.setPaymentMethod(paymentMethod);
        // 设置创建者
        SysUser sysUser = sysUserService.selectUserById(userId);
        userMember.setCreateBy(sysUser.getUserName());
        userMember.setCreateTime(new Date());
        userMember.setUpdateBy(sysUser.getUserName());
        userMember.setUpdateTime(new Date());

        baseMapper.insert(userMember);


        return userMember;
    }
}
