package com.mcwl.memberCenter.task;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.mcwl.common.constant.QueueConstants;
import com.mcwl.memberCenter.domain.UserMember;
import com.mcwl.memberCenter.enums.MemberMenu;
import com.mcwl.memberCenter.service.UserMemberService;
import lombok.RequiredArgsConstructor;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

@Component("userMemberTask")
@RequiredArgsConstructor
public class UserMemberTask {

    private final UserMemberService userMemberService;

    private final RabbitTemplate rabbitTemplate;


    /**
     * 积分清零提醒 月底前五天
     */
    public void emptyPointsRemindTask() {
        List<UserMember> userMemberList = this.getUseUserMember();
        if (userMemberList == null || userMemberList.isEmpty()) {
            return;
        }
        // 发送积分清零消息
        for (UserMember userMember : userMemberList) {
            rabbitTemplate.convertAndSend(QueueConstants.EMPTY_POINTS_REMIND_QUEUE, userMember);
        }

    }


    /**
     * 清空积分 每月月底前两天
     */
    public void emptyPointsTsk() {
        List<UserMember> userMemberList = this.getUseUserMember();
        if (userMemberList == null || userMemberList.isEmpty()) {
            return;
        }
        for (UserMember userMember : userMemberList) {
            userMember.setPoints(0);
            userMember.setUpdateTime(new Date());
        }
        userMemberService.updateBatchById(userMemberList);
    }

    /**
     * 更新会员状态 间隔1h检查
     */
    public void updateSubscriptionStatusTask() {
        LambdaQueryWrapper<UserMember> qw = new LambdaQueryWrapper<>();
        // endDate大于当前时间, subscriptionStatus不为过期
        qw.gt(UserMember::getEndDate, System.currentTimeMillis())
                .ne(UserMember::getSubscriptionStatus, MemberMenu.MEMBER_CENTER_EXPIRED)
                .ne(UserMember::getSubscriptionStatus, MemberMenu.MEMBER_CENTER_PENDING);
        List<UserMember> userMemberList = userMemberService.list(qw);
        if (userMemberList == null || userMemberList.isEmpty()) {
            return;
        }

        System.out.println("userMemberList = " + userMemberList);

        for (UserMember userMember : userMemberList) {
            MemberMenu subscriptionStatus = userMember.getSubscriptionStatus();
            if (subscriptionStatus == MemberMenu.MEMBER_CENTER_ACTIVE) {
                // 如果subscriptionStatus是活跃的，表示连续包月。订阅状态改为"待支付"
                userMember.setSubscriptionStatus(MemberMenu.MEMBER_CENTER_PENDING);
                // nextBillingDate设置为当前会员结束日期的下个月的同一天,发送账单通知
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(userMember.getEndDate());
                calendar.add(Calendar.MONTH, 1);
                userMember.setNextBillingDate(calendar.getTime());
                userMemberService.updateById(userMember);
                // 发送会员账单消息，如果支付成功，更新last_payment_date，并重新计算end_date（start_date + 1个月）
                rabbitTemplate.convertAndSend(QueueConstants.MEMBER_BILLING_QUEUE, userMember);
            } else if (subscriptionStatus == MemberMenu.MEMBER_CENTER_INACTIVE) {
                // 不是连续包月，会员状态改为过期，状态改为"1"
                userMember.setSubscriptionStatus(MemberMenu.MEMBER_CENTER_EXPIRED);
                userMember.setStatus("1");
                userMember.setUpdateTime(new Date());
            }
        }
    }


    // 获取正常使用的会员
    private List<UserMember> getUseUserMember() {
        // startDate 小于等于当前时间、endDate 大于等于当前时间
        // subscriptionStatus 不为 "过期" 或 "待支付"
        // status 为 0 的
        LambdaQueryWrapper<UserMember> qw = new LambdaQueryWrapper<>();
        qw.le(UserMember::getStartDate, new Date())
                .ge(UserMember::getEndDate, new Date())
                .ne(UserMember::getSubscriptionStatus, MemberMenu.MEMBER_CENTER_EXPIRED)
                .ne(UserMember::getSubscriptionStatus, MemberMenu.MEMBER_CENTER_PENDING)
                .eq(UserMember::getStatus, "0");
        // 对应的sql为
        System.out.println("sql = " + qw.getSqlSegment());
        return userMemberService.list(qw);
    }


}
